﻿//-----------------------------------------------------------------------
// <copyright file="CurrentObservations.cs" company="Eclectrics, LLC">
//     Copyright © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
//
// Free for all uses under the Creative Commons Attribution 3.0 United 
// Stated License.
//-----------------------------------------------------------------------

namespace Eclectrics.NationalWeatherServiceLibrary
{
    using System;
    using System.Diagnostics;
    using System.Drawing;
    using System.IO;
    using System.Net;
    using System.Text;

    /// <summary>
    /// Provides a set of operations dealing with NWS current observations.
    /// </summary>
    public static class CurrentObservations
    {
        /// <summary>
        /// Get the current observations from the requested observation station.
        /// </summary>
        /// <param name="stationId">station id of station for which current observations are requested</param>
        /// <param name="currentObservationDoc">if successful, Xml string containing current observations</param>
        /// <param name="status">description of result</param>
        /// <returns>true if successful, otherwise false</returns>
        public static bool GetCurrentXmlObservationFromStationId(string stationId, out string currentObservationDoc, out string status)
        {
            bool success = false;
            currentObservationDoc = null;

            if (string.IsNullOrEmpty(stationId))
            {
                status = "Failed to get current observation - stationId is invalid.";
                return false;
            }

            Debug.WriteLine("Requesting current observations for " + stationId + "  from National Weather Service.");

            HttpWebResponse response = null;
            StreamReader reader = null;
            try
            {
                var url = @"http://www.weather.gov/xml/current_obs/" + stationId + ".xml";
                var request = (HttpWebRequest)WebRequest.Create(url);
                response = (HttpWebResponse)request.GetResponse();

                var responseStream = response.GetResponseStream();
                if (responseStream != null)
                {
                    reader = new StreamReader(responseStream, Encoding.UTF8);
                    currentObservationDoc = reader.ReadToEnd();
                    status = Properties.Resources.Success;
                    success = true;
                }
                else
                {
                    status = Properties.Resources.Fail;
                }
            }
            catch (Exception ex)
            {
                currentObservationDoc = null;
                status = ex.Message;
            }
            finally
            {
                if (response != null)
                {
                    response.Close();
                }

                if (reader != null)
                {
                    reader.Close();
                }
            }

            return success;
        }

        /// <summary>
        /// Get the current observations from the observation station closest to the requested zip code.
        /// </summary>
        /// <param name="zipCode">Zip Code for which observations are requested</param>
        /// <param name="currentObservationDoc">if successful, Xml string containing current observations</param>
        /// <param name="stationName">station name</param>
        /// <param name="distance">distance to station</param>
        /// <param name="status">description of result</param>
        /// <returns>true if successful, otherwise false</returns>
        public static bool GetCurrentXmlObservationFromZip(string zipCode, out string currentObservationDoc, out string stationName, out double distance, out string status)
        {
            bool success = false;

            currentObservationDoc = string.Empty;

            PointF latLon;

            if (GetLatLonFromZip(zipCode, out latLon, out status))
            {
                var station = ObservationStations.GetClosest((decimal)latLon.X, (decimal)latLon.Y, out distance);
                if (station != null)
                {
                    stationName = station.StationName + ", " + station.StateAbbreviation + " (" + station.StationId + ")";
                    success = GetCurrentXmlObservationFromStationId(station.StationId, out currentObservationDoc, out status);
                }
                else
                {
                    stationName = "No station found";
                    distance = double.MaxValue;
                }
            }
            else
            {
                stationName = "Invalid Zip Code";
                distance = double.MaxValue;
            }

            return success;
        }

        /// <summary>
        /// Get the latitude and longitude corresponding to the requested zip code.
        /// </summary>
        /// <param name="zipString">zip code as a string</param>
        /// <param name="latLon">if successful, latitude and longitude, otherwise PointF.Empty</param>
        /// <param name="status">description of the result</param>
        /// <returns>true if successful, otherwise false</returns>
        private static bool GetLatLonFromZip(string zipString, out PointF latLon, out string status)
        {
            if (string.IsNullOrEmpty(zipString))
            {
                throw new ArgumentNullException("zipString");
            }

            bool success = false;

            latLon = new PointF();

            using (var proxy = new NdfdClient())
            {
                try
                {
                    if (proxy.Initialize(out status))
                    {
                        success = proxy.GetLonLatForZip(zipString, out latLon, out status);
                    }
                }
                catch (Exception ex)
                {
                    status = ex.Message;
                    success = false;
                }
            }

            return success;
        }
    }
}
