﻿//-----------------------------------------------------------------------
// <copyright file="NdfdClient.cs" company="Eclectrics, LLC">
//     Copyright © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
//
// Free for all uses under the Creative Commons Attribution 3.0 United 
// Stated License.
//-----------------------------------------------------------------------

namespace Eclectrics.NationalWeatherServiceLibrary
{
    using System;
    using System.Diagnostics;
    using System.Drawing;
    using System.ServiceModel;
    using System.ServiceModel.Channels;
    using System.Text;
    using System.Xml;

    using Eclectrics.NationalWeatherServiceLibrary.NdfdServiceReference;
    using SimpleTextMessageEncoderLibrary;

    /// <summary>
    /// Provides access to the NDFD web service.
    /// </summary>
    public class NdfdClient : IDisposable
    {
        /// <summary>
        /// The URL of the NDFD web service.
        /// </summary>
        private const string EndpointAddress = "http://graphical.weather.gov/xml/SOAP_server/ndfdXMLserver.php";

        /// <summary>
        /// The NDFD web service proxy
        /// </summary>
        private ndfdXMLPortType client;

        /// <summary>
        /// A channel factory that can be used to create a channel to the NDFD web service.
        /// </summary>
        private ChannelFactory<ndfdXMLPortType> channelFactory;

        /// <summary>
        /// Finalizes an instance of the NdfdClient class. This will run only if IDisposable.Dispos()e is not 
        /// called - insures unmanaged resource cleanup.
        /// </summary>
        ~NdfdClient()
        {
            // At this point, only cleanup unmanaged resources
            this.Dispose(false);
        }

        /// <summary>
        /// Open the channel.
        /// </summary>
        /// <param name="explaination">description of the result</param>
        /// <returns>true if successful, otherwise false</returns>
        public bool Initialize(out string explaination)
        {
            bool success;

            try
            {
                var binding = new CustomBinding();

                var encoder = new SimpleTextMessageEncoderBindingElement
                                                                    {
                                                                        Encoding = "ISO-8859-1",
                                                                        MessageVersion = MessageVersion.Soap11,
                                                                        MediaType = "text/xml"
                                                                    };
                binding.Elements.Add(encoder);

                var transport = new HttpTransportBindingElement
                                                            {
                                                                AllowCookies = false,
                                                                HostNameComparisonMode = HostNameComparisonMode.StrongWildcard
                                                            };
                binding.Elements.Add(transport);

                this.channelFactory = new ChannelFactory<ndfdXMLPortType>(binding, new EndpointAddress(EndpointAddress));
                this.channelFactory.Open();

                this.client = this.channelFactory.CreateChannel();

                explaination = "Success";
                success = true;
            }
            catch (Exception ex)
            {
                explaination = ex.Message;
                success = false;
            }

            return success;
        }

        /// <summary>
        /// Close the channel.
        /// </summary>
        public void Cleanup()
        {
            if (this.channelFactory != null && this.channelFactory.State == CommunicationState.Opened)
            {
                this.channelFactory.Close();
            }
        }

        /// <summary>
        /// Releases unmanaged resources
        /// </summary>
        void IDisposable.Dispose()
        {
            this.Dispose(true);

            // Take this object off the finalization queue.
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Get the longitude/latitude for a US Zip code using the NDFD Web service.
        /// </summary>
        /// <param name="zip">zip of requested point</param>
        /// <param name="lonlatPoint">if successful, latitude and longitude of the point corresponding
        /// to the supplies zip code</param>
        /// <param name="explaination">description of result</param>
        /// <returns>true if successful, otherwise false</returns>
        public bool GetLonLatForZip(string zip, out PointF lonlatPoint, out string explaination)
        {
            bool success = false;
            lonlatPoint = Point.Empty;  // (0.0, 0.0) is not a valid point in the US

            if (this.client == null && !this.Initialize(out explaination))
            {
                return false;
            }

            var doc = new XmlDocument();

            try
            {
                if (this.client != null)
                {
                    doc.LoadXml(this.client.LatLonListZipCode(zip));
                }
                else
                {
                    explaination = "Failed to create proxy for NDFD web service.";
                    return false;
                }
            }
            catch (CommunicationException cex)
            {
                explaination = "Failed to get location: Bad Zip Code: " + cex.Message;
                return false;
            }
            catch (Exception ex)
            {
                explaination = "Failed to get location: " + ex.Message;
                return false;
            }

            XmlElement root = doc.DocumentElement;
            if (root == null)
            {
                explaination = "NDFD web service did not return a valid location document";
                return false;
            }

            XmlNode latLonNode = root.SelectSingleNode("latLonList");

            if (latLonNode != null)
            {
                var separators = new[] { ',' };
                var lonLatStrArray = latLonNode.InnerText.Split(separators, StringSplitOptions.None);

                if (lonLatStrArray.Length == 2)
                {
                    float latitude;
                    if (float.TryParse(lonLatStrArray[0], out latitude))
                    {
                        float longitude;
                        success = float.TryParse(lonLatStrArray[1], out longitude);

                        if (success)
                        {
                            lonlatPoint = new PointF(longitude, latitude);
                            explaination = "Success";
                        }
                        else
                        {
                            explaination = "Could not parse returned latitude string.";
                        }
                    }
                    else
                    {
                        explaination = "Could not parse returned longitude string.";
                    }
                }
                else
                {
                    explaination = "Parsed XML did not contain one longitude and one latitude string.";
                }
            }
            else
            {
                explaination = "Returned XML did not contain a latLonList.";
            }

            return success;
        }

        /// <summary>
        /// Get a weather forecast for the requested location and number of days from the NWS NDFD.
        /// </summary>
        /// <param name="longitude">longitude for forecast</param>
        /// <param name="latitude">latitude for forecast</param>
        /// <param name="days">number of days in forecast</param>
        /// <returns>an Xml string containing the forecast</returns>
        public string GetXmlForecastFromLatLon(decimal longitude, decimal latitude, string days)
        {
            string result;

            try
            {
                result = this.client.NDFDgenByDay(
                    latitude,
                    longitude,
                    DateTime.Now,
                    days,
                    unitType.e, 
                    formatType.Item12hourly);

                Debug.WriteLine(result);
            }
            catch (CommunicationException comEx)
            {
                var text = new StringBuilder();
                text.AppendLine("Web Service Error: ");
                text.AppendLine(comEx.InnerException.Message);
                result = text.ToString();
            }
            catch (Exception ex)
            {
                var text = new StringBuilder();
                text.AppendLine("Error: ");
                text.AppendLine(ex.Message);
                result = text.ToString();
            }

            return result;
        }

        /// <summary>
        /// Called by user code (disposingAll = true) and from the instance destructor (disposingAll = false)
        /// to signal that we should cleanup before being destroyed.
        /// </summary>
        /// <param name="disposingAll">true if called from user code, otherwise false</param>
        private void Dispose(bool disposingAll)
        {
            // If called from user code, cleanup up managed resources.
            if (disposingAll)
            {
                this.Cleanup();
            }

            // Always cleanup unmanaged resources.
        }
    }
}
