﻿//-----------------------------------------------------------------------
// <copyright file="ObservationStation.cs" company="Eclectrics, LLC">
//     Copyright © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
//
// Free for all uses under the Creative Commons Attribution 3.0 United 
// Stated License.
//-----------------------------------------------------------------------

namespace Eclectrics.NationalWeatherServiceLibrary
{
    using System.Text;
    using System.Xml;

    /// <summary>
    /// Encapsulates information about an observation station.
    /// </summary>
    public class ObservationStation
    {
        /// <summary>
        /// Initializes a new instance of the ObservationStation class.
        /// </summary>
        public ObservationStation() 
        {
        }

        /// <summary>
        /// Initializes a new instance of the ObservationStation class from 
        /// the supplied XmlNode.
        /// </summary>
        /// <param name="stationNode">XmlNode describing the station</param>
        public ObservationStation(XmlNode stationNode)
        {
            this.InitializeFromXmlNode(stationNode);
        }

        /// <summary>
        /// Gets or sets the full state name.
        /// </summary>
        public string StateName { get; set; }

        /// <summary>
        /// Gets or sets the 2-letter state abbreviation.
        /// </summary>
        public string StateAbbreviation { get; set; }

        /// <summary>
        /// Gets or sets the station name.
        /// </summary>
        public string StationName { get; set; }

        /// <summary>
        /// Gets or sets the station id.
        /// </summary>
        public string StationId { get; set; }

        /// <summary>
        /// Gets or sets the URL of web page containing current observations.
        /// </summary>
        public string CurrentObsWebPageUrl { get; set; }

        /// <summary>
        /// Gets or sets URL of current observations RSS feed.
        /// </summary>
        public string CurrentObsRssUrl { get; set; }

        /// <summary>
        /// Gets or sets the URL of current observations XML document.
        /// </summary>
        public string CurrentObsXmlUrl { get; set; }

        /// <summary>
        /// Gets or sets the observation station latitude.
        /// </summary>
        public decimal Latitude { get; set; }

        /// <summary>
        /// Gets or sets the observation station longitude.
        /// </summary>
        public decimal Longitude { get; set; }

        /// <summary>
        /// This allows us to add ObservationStations as items in a ComboBox
        /// and have them display as the name of the station.
        /// </summary>
        /// <returns>name of the observation station</returns>
        public override string ToString()
        {
            return this.StationName;
        }

        /// <summary>
        /// Convert instance data to a full-details string.
        /// </summary>
        /// <returns>a multi-line string containing all the station info</returns>
        public string ToFullString()
        {
            var text = new StringBuilder();
            text.AppendLine("Station name: " + this.StationName);
            text.AppendLine("Station id: " + this.StationId);
            text.AppendLine("Location: " + this.Longitude + ", " + this.Latitude);
            text.AppendLine("Html Web page: " + this.CurrentObsWebPageUrl);
            text.AppendLine("Xml Web page: " + this.CurrentObsXmlUrl);
            text.AppendLine("RSS Web page: " + this.CurrentObsRssUrl);
            text.AppendLine("State: " + this.StateName + " (" + this.StateAbbreviation + ")");
            return text.ToString();
        }

        /// <summary>
        /// Convert instance data to a medium-details string.
        /// </summary>
        /// <returns>a multi-line string  containing the station name, state and station id</returns>
        public string ToMediumString()
        {
            var text = new StringBuilder();
            text.AppendLine("Station name: " + this.StationName + ", " + this.StateName);
            text.AppendLine("Station id: " + this.StationId);
            return text.ToString();
        }

        /// <summary>
        /// Convert instance data to a brief string.
        /// </summary>
        /// <returns>a single-line string containing the station and and id</returns>
        public string ToShortString()
        {
            return this.StationName + ", " + this.StationId;
        }

        /// <summary>
        /// Initialize the ObservationStation property values from the supplied Xml element.
        /// </summary>
        /// <param name="stationNode">XmlNode describing the observation station</param>
        private void InitializeFromXmlNode(XmlNode stationNode)
        {
            var stateNames = new StateNames();

            if (stationNode.HasChildNodes)
            {
                var stateNode = stationNode.SelectSingleNode("state");
                if (stateNode != null)
                {
                    this.StateAbbreviation = stateNode.InnerText;
                    this.StateName = stateNames.Name(this.StateAbbreviation);
                }
                
                var nameNode = stationNode.SelectSingleNode("station_name");
                if (nameNode != null)
                {
                    this.StationName = nameNode.InnerText;
                }

                var idNode = stationNode.SelectSingleNode("station_id");
                if (idNode != null)
                {
                    this.StationId = idNode.InnerText;
                }

                var xmlUrlNode = stationNode.SelectSingleNode("xml_url");
                if (xmlUrlNode != null)
                {
                    this.CurrentObsXmlUrl = xmlUrlNode.InnerText;
                }

                var rssUrlNode = stationNode.SelectSingleNode("rss_url");
                if (rssUrlNode != null)
                {
                    this.CurrentObsRssUrl = rssUrlNode.InnerText;
                }

                var htmlUrlNode = stationNode.SelectSingleNode("html_url");
                if (htmlUrlNode != null)
                {
                    this.CurrentObsWebPageUrl = htmlUrlNode.InnerText;
                }

                var latNode = stationNode.SelectSingleNode("latitude");
                if (latNode != null)
                {
                    decimal latitude;
                    if (decimal.TryParse(latNode.InnerText, out latitude))
                    {
                        this.Latitude = latitude;
                    }
                }
                
                var lonNode = stationNode.SelectSingleNode("longitude");
                if (lonNode != null)
                {
                    decimal longitude;
                    if (decimal.TryParse(lonNode.InnerText, out longitude))
                    {
                        this.Longitude = longitude;
                    }
                }
            }
        }
    }
}
