﻿//-----------------------------------------------------------------------
// <copyright file="ObservationStations.cs" company="Eclectrics, LLC">
//     Copyright © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
//
// Free for all uses under the Creative Commons Attribution 3.0 United 
// Stated License.
//-----------------------------------------------------------------------

namespace Eclectrics.NationalWeatherServiceLibrary
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.IO;
    using System.IO.IsolatedStorage;
    using System.Net;
    using System.Text;
    using System.Xml;

    /// <summary>
    /// Provides a set of operations dealing with the collection of NWS
    /// ObservationStations.
    /// </summary>
    public static class ObservationStations
    {
        /// <summary>
        /// File name in IsolatedStorage where the Xml document containing
        /// the data on NWS observation stations is stored.
        /// </summary>
        private const string ObservationStationsFile = "index.xml";

        /// <summary>
        /// The radius of the earth, used for computing the distance from
        /// a specific latitude and longitude to a observation station.
        /// </summary>
        private const double EarthRadius = 6378.16; // in km

        /// <summary>
        /// Gets a value indicating whether we have a cache of observation stations.
        /// </summary>
        public static bool HaveStations
        {
            get
            {
                bool have = false;

                try
                {
                    var isoStore = IsolatedStorageFile.GetStore(IsolatedStorageScope.Machine | IsolatedStorageScope.Assembly, null, null);
                    have = isoStore.GetFileNames(ObservationStationsFile).Length > 0;
                }
                catch (Exception ex)
                {
                    Debug.WriteLine("HaveStations error: " + ex.Message);
                }

                return have;
            }
        }

        /// <summary>
        /// Get a collection of all states for which we can locate observation stations.
        /// </summary>
        /// <returns>the collection of all states</returns>
        public static IList<StateInfo> GetStates()
        {
            var states = new SortedList<string, StateInfo>();

            using (var reader = new StreamReader("StateNames.xml"))
            {
                var doc = new XmlDocument();
                doc.Load(reader);

                if (doc.DocumentElement != null)
                {
                    var stateNodes = doc.DocumentElement.SelectNodes("state");
                    if (stateNodes != null)
                    {
                        foreach (XmlNode stateNode in stateNodes)
                        {
                            if (stateNode.Attributes != null)
                            {
                                string stateName = stateNode.Attributes["name"].InnerText;
                                string stateAbbr = stateNode.Attributes["abbr"].InnerText;
                                if (!string.IsNullOrEmpty(stateName) &&
                                    !string.IsNullOrEmpty(stateAbbr) &&
                                    !states.ContainsKey(stateAbbr))
                                {
                                    states.Add(stateAbbr, new StateInfo(stateName, stateAbbr));
                                }
                            }
                        }
                    }
                }
            }

            return states.Values;
        }

        /// <summary>
        /// Get the collection of observation stations located in the requested state.
        /// </summary>
        /// <param name="stateAbbreviation">standard 2-letter abbreviation of state for
        /// which observation stations are requested</param>
        /// <param name="stations">if successful, requested observation stations, otherwise null</param>
        /// <param name="status">describes result</param>
        /// <returns>true if successful, otherwise false</returns>
        public static bool GetStations(string stateAbbreviation, out IList<ObservationStation> stations, out string status)
        {
            bool success = false;
            var stationsList = new SortedList<string, ObservationStation>();

            XmlDocument doc;
            if (GetObservationStationsXmlDocument(out doc, out status))
            {
                try
                {
                    if (doc.DocumentElement != null && doc.DocumentElement.HasChildNodes)
                    {
                        var xmlStations = doc.DocumentElement.SelectNodes("station");

                        if (xmlStations != null)
                        {
                            foreach (XmlNode xmlStation in xmlStations)
                            {
                                if (xmlStation.HasChildNodes)
                                {
                                    var stateNode = xmlStation.SelectSingleNode("state");
                                    if (stateNode != null && !string.IsNullOrEmpty(stateNode.InnerText) && stateNode.InnerText.Equals(stateAbbreviation))
                                    {
                                        var station = new ObservationStation(xmlStation);

                                        string key = GetStationKey(station);

                                        Debug.Assert(!stationsList.ContainsKey(key), "Duplicate Station");

                                        if (!stationsList.ContainsKey(key))
                                        {
                                            stationsList.Add(key, station);
                                        }
                                    }
                                }
                            }

                            success = true;
                            status = Properties.Resources.Success;
                        }
                        else
                        {
                            status = Properties.Resources.NWS_Bad_Stations_Data;
                        }
                    }
                    else
                    {
                        status = Properties.Resources.NWS_Bad_Stations_Data;
                    }
                }
                catch (Exception ex)
                {
                    status = ex.Message;
                }
            }

            stations = stationsList.Values;
            return success;
        }

        /// <summary>
        /// A brute force great-circle computation to find the closest NWS observation
        /// station.
        /// </summary>
        /// <param name="longitude">longitude for evaluation</param>
        /// <param name="latitude">latitude for evaluation</param>
        /// <param name="minDistance">The min distance.</param>
        /// <returns>
        /// closest observation station
        /// </returns>
        public static ObservationStation GetClosest(decimal longitude, decimal latitude, out double minDistance)
        {
            ObservationStation closestStation = null;
            minDistance = double.MaxValue;
            IList<ObservationStation> stations;

            string status;
            if (GetStations(out stations, out status) && stations != null)
            {
                foreach (ObservationStation station in stations)
                {
                    double distance = GetDistance((double)latitude, (double)longitude, (double)station.Latitude, (double)station.Longitude);

                    if (distance < minDistance)
                    {
                        closestStation = station;
                        minDistance = distance;
                    }
                }
            }

            return closestStation;
        }

        /// <summary>
        /// We cache a local copy of the NWS Observation stations list available 
        /// from: http://www.weather.gov/xml/current_obs/index.xml. This method
        /// gets a new copy from the web site and updates the local cache.
        /// </summary>
        /// <param name="status">description of result</param>
        /// <returns>true if successful, otherwise false</returns>
        public static bool FlushObservationFileCache(out string status)
        {
            bool success = false;

            try
            {
                IsolatedStorageFile isoStore =
                    IsolatedStorageFile.GetStore(IsolatedStorageScope.Machine | IsolatedStorageScope.Assembly, null, null);

                string[] fileNames = isoStore.GetFileNames(ObservationStationsFile);
                if (fileNames.Length > 0)
                {
                    try
                    {
                        isoStore.DeleteFile(ObservationStationsFile);
                        status = Properties.Resources.Success;
                        success = true;
                    }
                    catch (Exception ex)
                    {
                        status = ex.Message;
                    }
                }
                else
                {
                    status = Properties.Resources.No_ObservationStationsFile_Cache;
                }
            }
            catch (Exception ex)
            {
                status = ex.Message;
            }

            return success;
        }

        /// <summary>
        /// Gets the collection of all observation stations.
        /// </summary>
        /// <param name="stations">if successful, the collection of all observation stations,
        /// otherwise null</param>
        /// <param name="status">description of result</param>
        /// <returns>true if successful, otherwise false</returns>
        private static bool GetStations(out IList<ObservationStation> stations, out string status)
        {
            bool success = false;
            var stationsList = new SortedList<string, ObservationStation>();

            XmlDocument doc;
            if (GetObservationStationsXmlDocument(out doc, out status))
            {
                try
                {
                    if (doc.DocumentElement != null && doc.DocumentElement.HasChildNodes)
                    {
                        var xmlStations = doc.DocumentElement.SelectNodes("station");

                        if (xmlStations != null)
                        {
                            foreach (XmlNode xmlStation in xmlStations)
                            {
                                var station = new ObservationStation(xmlStation);
                                if (!stationsList.ContainsKey(station.StationId))
                                {
                                    stationsList.Add(station.StationId, station);
                                }
                            }

                            success = true;
                            status = Properties.Resources.Success;
                        }
                        else
                        {
                            status = Properties.Resources.NWS_Bad_Stations_Data;
                        }
                    }
                    else
                    {
                        status = Properties.Resources.NWS_Bad_Stations_Data;
                    }
                }
                catch (Exception ex)
                {
                    status = ex.Message;
                }
            }

            stations = stationsList.Values;
            return success;
        }

        /// <summary>
        /// Make a unique string from an ObservationStation that can be used for 
        /// alphabetic sorting. There is some duplication of station names so we
        /// append the station Id to create a unique string for each station, that
        /// will still be sorted by station name in our list of stations.
        /// </summary>
        /// <param name="station">ObservationStation for which key is requested</param>
        /// <returns>a unique key for the station</returns>
        private static string GetStationKey(ObservationStation station)
        {
            // These are all signs that something is really wrong...
            if (station == null || station.StationName == null || station.StationId == null)
            {
                return string.Empty;
            }

            // Sort by station name and Id just in case there are two stations
            // with the same name (e.g., Newport, OR).
            return station.StationName + ": " + station.StationId;
        }

        /// <summary>
        /// A quick and dirty great-circle distance computation using the
        /// spherical law of cosines. You can find a nice description here:
        /// http://www.movable-type.co.uk/scripts/latlong.html
        /// </summary>
        /// <param name="lat1">point1 latitude</param>
        /// <param name="lon1">point1 longitude</param>
        /// <param name="lat2">point2 latitude</param>
        /// <param name="lon2">point2 longitude</param>
        /// <returns>distance in km.</returns>
        private static double GetDistance(double lat1, double lon1, double lat2, double lon2)
        {
            double radLat1 = ToRadians(lat1);
            double radLon1 = ToRadians(lon1);
            double radLat2 = ToRadians(lat2);
            double radLon2 = ToRadians(lon2);
            return Math.Acos((Math.Sin(radLat1) * Math.Sin(radLat2)) + 
                             (Math.Cos(radLat1) * Math.Cos(radLat2) * 
                              Math.Cos(radLon2 - radLon1))) * EarthRadius;
        }

        /// <summary>
        /// Convert an angle in (decimal) degrees to radians.
        /// </summary>
        /// <param name="degrees">angle in decimal degrees</param>
        /// <returns>angle in radians</returns>
        private static double ToRadians(double degrees)
        {
            return (degrees * Math.PI) / 180;
        }

        /// <summary>
        /// Get the XmlDocument containing the description of all ObservationStations.
        /// If the document exists in IsolatedStorage, it is loaded and returned, otherwise
        /// the document is requested from the NWS web site.
        /// </summary>
        /// <param name="observationStationsDoc">the XmlDocument describing all ObservationStations</param>
        /// <param name="status">description of result</param>
        /// <returns>true if successful, otherwise false</returns>
        private static bool GetObservationStationsXmlDocument(out XmlDocument observationStationsDoc, out string status)
        {
            bool success;

            if (!(success = GetStationsDocFromIsoStorage(out observationStationsDoc, out status)))
            {
                success = GetStationsDocFromNws(out observationStationsDoc, out status);

                if (success)
                {
                    success = SaveStationsDocToIsoStorage(observationStationsDoc, out status);
                }
            }

            return success;
        }

        /// <summary>
        /// Attempts to retrieve the XmlDocument describing all NWS ObservationStations
        /// from IsolatedStorage.
        /// </summary>
        /// <param name="observationStationsDoc">if successful, the XmlDocument describing 
        /// all NWS ObservationStations, otherwise null</param>
        /// <param name="status">description of result</param>
        /// <returns>true if successful, otherwise false</returns>
        private static bool GetStationsDocFromIsoStorage(out XmlDocument observationStationsDoc, out string status)
        {
            bool success = false;

            try
            {
                var isoStore = IsolatedStorageFile.GetStore(IsolatedStorageScope.Machine | IsolatedStorageScope.Assembly, null, null);

                string[] fileNames = isoStore.GetFileNames(ObservationStationsFile);
                if (fileNames.Length > 0)
                {
                    try
                    {
                        using (var reader = new StreamReader(new IsolatedStorageFileStream(ObservationStationsFile, FileMode.Open, isoStore)))
                        {
                            observationStationsDoc = new XmlDocument();
                            observationStationsDoc.Load(reader);
                        }

                        success = true;
                        status = Properties.Resources.Success;
                    }
                    catch (Exception ex)
                    {
                        observationStationsDoc = null;
                        status = ex.Message;
                    }
                }
                else
                {
                    observationStationsDoc = null;
                    status = Properties.Resources.No_Stations_Doc;
                }
            }
            catch (Exception ex)
            {
                observationStationsDoc = null;
                status = ex.Message;
            }

            return success;
        }

        /// <summary>
        /// Attempts to retrieve the XmlDocument describing all NWS ObservationStations
        /// from the NWS web site.
        /// </summary>
        /// <param name="observationStationsDoc">if successful, the XmlDocument describing 
        /// all NWS ObservationStations, otherwise null</param>
        /// <param name="status">description of result</param>
        /// <returns>true if successful, otherwise false</returns>
        private static bool GetStationsDocFromNws(out XmlDocument observationStationsDoc, out string status)
        {
            bool success = false;

            Debug.WriteLine("Downloading Observation Stations document from National Weather Service.");

            HttpWebResponse response = null;
            StreamReader reader = null;
            try
            {
                var request = (HttpWebRequest)WebRequest.Create("http://www.weather.gov/xml/current_obs/index.xml");
                response = (HttpWebResponse)request.GetResponse();

                var responseStream = response.GetResponseStream();
                if (responseStream != null)
                {
                    reader = new StreamReader(responseStream, Encoding.UTF8);
                    observationStationsDoc = new XmlDocument();
                    observationStationsDoc.Load(reader);
                    status = Properties.Resources.Success;
                    success = true;
                }
                else
                {
                    status = Properties.Resources.Fail;
                    observationStationsDoc = null;
                }
            }
            catch (Exception ex)
            {
                observationStationsDoc = null;
                status = ex.Message;
            }
            finally
            {
                if (response != null)
                {
                    response.Close();
                }

                if (reader != null)
                {
                    reader.Close();
                }
            }

            return success;
        }

        /// <summary>
        /// Saves the supplied XmlDocument to IsolatedStorage.
        /// </summary>
        /// <param name="observationStationsDoc">XmlDocument describing 
        /// all NWS ObservationStations</param>
        /// <param name="status">description of result</param>
        /// <returns>true if successful, otherwise false</returns>
        private static bool SaveStationsDocToIsoStorage(XmlDocument observationStationsDoc, out string status)
        {
            bool success = false;

            try
            {
                var isoStore = IsolatedStorageFile.GetStore(IsolatedStorageScope.Machine | IsolatedStorageScope.Assembly, null, null);

                using (var writer = new StreamWriter(new IsolatedStorageFileStream(ObservationStationsFile, FileMode.OpenOrCreate, isoStore)))
                {
                    observationStationsDoc.Save(writer);
                }

                success = true;
                status = Properties.Resources.Success;
            }
            catch (Exception ex)
            {
                status = ex.Message;
            }

            return success;
        }
    }
}
