﻿//-----------------------------------------------------------------------
// <copyright file="SimpleTextMessageEncoder.cs" company="Eclectrics, LLC">
//     Copyright © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
//
// The intellectual property contained in this class belongs to Microsoft.
//  
// This document is free for all uses under the Creative Commons Attribution 
// 3.0 United Stated License.
//-----------------------------------------------------------------------

// This code is from the Microsoft WCF sample that can be found here:
// http://msdn.microsoft.com/en-us/library/ms751486(VS.85).aspx

namespace SimpleTextMessageEncoderLibrary
{
    using System;
    using System.IO;
    using System.ServiceModel.Channels;
    using System.Text;
    using System.Xml;

    /// <summary>
    /// A message encoder using the ISO-8859-1 encoding.
    /// </summary>
    internal class SimpleTextMessageEncoder : MessageEncoder
    {
        /// <summary>
        /// The factory that creates instances of this type.
        /// </summary>
        private readonly SimpleTextMessageEncoderFactory factory;

        /// <summary>
        /// The XML writer settings
        /// </summary>
        private readonly XmlWriterSettings writerSettings;

        /// <summary>
        /// A string describing the media type and encoding.
        /// </summary>
        private readonly string contentType;

        /// <summary>
        /// Initializes a new instance of the <see cref="SimpleTextMessageEncoder"/> class.
        /// </summary>
        /// <param name="factory">The factory.</param>
        public SimpleTextMessageEncoder(SimpleTextMessageEncoderFactory factory)
        {
            this.factory = factory;
            this.writerSettings = new XmlWriterSettings {Encoding = Encoding.GetEncoding(factory.CharSet)};
            this.contentType = string.Format("{0}; charset={1}", this.factory.MediaType, this.writerSettings.Encoding.HeaderName);
        }

        /// <summary>
        /// When overridden in a derived class, gets the MIME content type used by the encoder.
        /// </summary>
        /// <returns>The content type that is supported by the message encoder.</returns>
        public override string ContentType { get { return this.contentType; } }

        /// <summary>
        /// When overridden in a derived class, gets the media type value that is used by the encoder.
        /// </summary>
        /// <returns>The media type that is supported by the message encoder.</returns>
        public override string MediaType { get { return this.factory.MediaType; } }

        /// <summary>
        /// When overridden in a derived class, gets the message version value that is used by the encoder.
        /// </summary>
        /// <returns>The <see cref="T:System.ServiceModel.Channels.MessageVersion"/> that is used by the encoder.</returns>
        public override MessageVersion MessageVersion { get { return this.factory.MessageVersion; } }

        /// <summary>
        /// When overridden in a derived class, reads a message from a specified stream.
        /// </summary>
        /// <param name="buffer">A <see cref="T:System.ArraySegment`1"/> of type <see cref="T:System.Byte"/> that provides the buffer from which the message is deserialized.</param>
        /// <param name="bufferManager">The <see cref="T:System.ServiceModel.Channels.BufferManager"/> that manages the buffer from which the message is deserialized.</param>
        /// <param name="messageContentType">The Multipurpose Internet Mail Extensions (MIME) message-level content-type.</param>
        /// <returns>
        /// The <see cref="T:System.ServiceModel.Channels.Message"/> that is read from the stream specified.
        /// </returns>
        public override Message ReadMessage(ArraySegment<byte> buffer, BufferManager bufferManager, string messageContentType)
        {
            var msgContents = new byte[buffer.Count];
            Array.Copy(buffer.Array, buffer.Offset, msgContents, 0, msgContents.Length);
            bufferManager.ReturnBuffer(buffer.Array);

            var stream = new MemoryStream(msgContents);
            return ReadMessage(stream, int.MaxValue);
        }

        /// <summary>
        /// When overridden in a derived class, reads a message from a specified stream.
        /// </summary>
        /// <param name="stream">The <see cref="T:System.IO.Stream"/> object from which the message is read.</param>
        /// <param name="maxSizeOfHeaders">The maximum size of the headers that can be read from the message.</param>
        /// <param name="messageContentType">The Multipurpose Internet Mail Extensions (MIME) message-level content-type.</param>
        /// <returns>
        /// The <see cref="T:System.ServiceModel.Channels.Message"/> that is read from the stream specified.
        /// </returns>
        public override Message ReadMessage(Stream stream, int maxSizeOfHeaders, string messageContentType)
        {
            var reader = XmlReader.Create(stream);
            var msgVersion = (this.MessageVersion ?? MessageVersion.None);
            return Message.CreateMessage(reader, maxSizeOfHeaders, msgVersion);
        }

        /// <summary>
        /// When overridden in a derived class, writes a message of less than a specified size to a byte array buffer at the specified offset.
        /// </summary>
        /// <param name="message">The <see cref="T:System.ServiceModel.Channels.Message"/> to write to the message buffer.</param>
        /// <param name="maxMessageSize">The maximum message size that can be written.</param>
        /// <param name="bufferManager">The <see cref="T:System.ServiceModel.Channels.BufferManager"/> that manages the buffer to which the message is written.</param>
        /// <param name="messageOffset">The offset of the segment that begins from the start of the byte array that provides the buffer.</param>
        /// <returns>
        /// A <see cref="T:System.ArraySegment`1"/> of type byte that provides the buffer to which the message is serialized.
        /// </returns>
        public override ArraySegment<byte> WriteMessage(Message message, int maxMessageSize, BufferManager bufferManager, int messageOffset)
        {
            byte[] messageBytes;
            int messageLength;
            using (var stream = new MemoryStream())
            {
                using (var writer = XmlWriter.Create(stream, this.writerSettings))
                {
                    message.WriteMessage(writer);
                    writer.Close();						
                }

                messageBytes = stream.GetBuffer();
                messageLength = (int) stream.Position;					
                stream.Close();
            }

            int totalLength = messageLength + messageOffset;
            byte[] totalBytes = bufferManager.TakeBuffer(totalLength);
            Array.Copy(messageBytes, 0, totalBytes, messageOffset, messageLength);

            var byteArray = new ArraySegment<byte>(totalBytes, messageOffset, messageLength);
            return byteArray;
        }

        /// <summary>
        /// When overridden in a derived class, writes a message to a specified stream.
        /// </summary>
        /// <param name="message">The <see cref="T:System.ServiceModel.Channels.Message"/> to write to the <paramref name="stream"/>.</param>
        /// <param name="stream">The <see cref="T:System.IO.Stream"/> object to which the <paramref name="message"/> is written.</param>
        public override void WriteMessage(Message message, Stream stream)
        {
            XmlWriter writer = XmlWriter.Create(stream, this.writerSettings);
            message.WriteMessage(writer);
            writer.Close();				
        }
    }
}
