﻿//-----------------------------------------------------------------------
// <copyright file="SimpleTextMessageEncoderBindingElement.cs" company="Eclectrics, LLC">
//     Copyright © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
//
// The intellectual property contained in this class belongs to Microsoft.
//  
// This document is free for all uses under the Creative Commons Attribution 
// 3.0 United Stated License.
//-----------------------------------------------------------------------

// This class uses code described here:
// http://msdn.microsoft.com/en-us/library/ms751486(VS.85).aspx


using System;
using System.Xml;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;


namespace SimpleTextMessageEncoderLibrary
{
    /// <summary>
    /// The binding element that specifies the message version used by our encoder.
    /// </summary>
    public class SimpleTextMessageEncoderBindingElement : MessageEncodingBindingElement, IWsdlExportExtension
    {
        /// <summary>
        /// The message version.
        /// </summary>
        private MessageVersion messageVersion;

        /// <summary>
        /// The media type.
        /// </summary>
        private string mediaType;

        /// <summary>
        /// The encoding.
        /// </summary>
        private string encoding;

        /// <summary>
        /// Constrains the complexity of the SOAP messages that can be processed
        /// by endpoints configured with this binding.
        /// </summary>
        private readonly XmlDictionaryReaderQuotas readerQuotas; 

        /// <summary>
        /// Initializes a new instance of the SimpleTextMessageEncoderBindingElement class.
        /// </summary>
        /// <param name="encodingParam">the encoding</param>
        /// <param name="mediaTypeParam">the media type</param>
        /// <param name="messageVersionParam">the MessageVersion</param>
        public SimpleTextMessageEncoderBindingElement(string encodingParam, string mediaTypeParam, MessageVersion messageVersionParam)
        {
            if (encodingParam == null)
            {
                throw new ArgumentNullException("encodingParam");
            }

            if (mediaTypeParam == null)
            {
                throw new ArgumentNullException("mediaTypeParam");
            }

            if (messageVersionParam == null)
            {
                throw new ArgumentNullException("messageVersionParam");
            }

            this.messageVersion = messageVersionParam;
            this.mediaType = mediaTypeParam;
            this.encoding = encodingParam;
            this.readerQuotas = new XmlDictionaryReaderQuotas();
        }

        /// <summary>
        /// Initializes a new instance of the SimpleTextMessageEncoderBindingElement class.
        /// </summary>
        /// <param name="encodingParam">the encoding</param>
        /// <param name="mediaTypeParam">the media type</param>
        public SimpleTextMessageEncoderBindingElement(string encodingParam, string mediaTypeParam) 
            : this(encodingParam, mediaTypeParam, MessageVersion.Soap11WSAddressing10)
        {
        }

        /// <summary>
        /// Initializes a new instance of the SimpleTextMessageEncoderBindingElement class.
        /// </summary>
        /// <param name="encodingParam">the encoding</param>
        public SimpleTextMessageEncoderBindingElement(string encodingParam)
            : this(encodingParam, "text.xml")
        {
        }

        /// <summary>
        /// Initializes a new instance of the SimpleTextMessageEncoderBindingElement class.
        /// </summary>
        public SimpleTextMessageEncoderBindingElement()
            : this("UTF-8")
        {
        }

        /// <summary>
        /// Initializes a new instance of the SimpleTextMessageEncoderBindingElement class.
        /// </summary>
        /// <param name="binding">the binding used to initialize the new instance</param>
        private SimpleTextMessageEncoderBindingElement(SimpleTextMessageEncoderBindingElement binding)
            : this(binding.Encoding, binding.MediaType, binding.MessageVersion)
        {
            this.readerQuotas = new XmlDictionaryReaderQuotas();
            binding.ReaderQuotas.CopyTo(this.readerQuotas);
        }

        /// <summary>
        /// Gets or sets the MessageVersion used to encode messages.
        /// </summary>
        public override MessageVersion MessageVersion
        {
            get { return messageVersion; }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                
                messageVersion = value;
            }
        }

        /// <summary>
        /// Gets or sets the media type used to format messages.
        /// </summary>
        public string MediaType
        {
            get { return mediaType; }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }

                mediaType = value;
            }
        }

        /// <summary>
        /// Gets or sets the encoding that is used to format the characters in the text message.
        /// </summary>
        public string Encoding
        {
            get { return encoding; }

            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }

                encoding = value;
            }
        }

        /// <summary>
        /// Gets the constraints on the complexity of SOAP messages that can be 
        /// processed by endpoints configured with this binding.
        /// </summary>
        public XmlDictionaryReaderQuotas ReaderQuotas 
        { 
            get { return readerQuotas; } 
        }

        #region IMessageEncodingBindingElement Members

        /// <summary>
        /// Creates a factory for simple text message encoders that employ the message version 
        /// and character encoding specified by the current encoding binding element. 
        /// </summary>
        /// <returns>the SimpleTextMessageEncoderFactory this factory creates</returns>
        public override MessageEncoderFactory CreateMessageEncoderFactory()
        {
            return new SimpleTextMessageEncoderFactory(this.MediaType, this.Encoding, this.MessageVersion);
        }

        #endregion

        /// <summary>
        /// Returns a copy of the binding element.
        /// </summary>
        /// <returns></returns>
        public override BindingElement Clone()
        {
            return new SimpleTextMessageEncoderBindingElement(this);
        }

        /// <summary>
        /// Initializes a channel factory for producing channels from the binding context.
        /// </summary>
        /// <typeparam name="TChannel">the type of channel the factory builds</typeparam>
        /// <param name="context">the context that provides the context for the binding element</param>
        /// <returns>the IChannelFactory of type TChannel initialized from the context</returns>
        public override IChannelFactory<TChannel> BuildChannelFactory<TChannel>(BindingContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }

            context.BindingParameters.Add(this);
            return context.BuildInnerChannelFactory<TChannel>();
        }

        public override bool CanBuildChannelFactory<TChannel>(BindingContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }

            return context.CanBuildInnerChannelFactory<TChannel>();
        }

        public override IChannelListener<TChannel> BuildChannelListener<TChannel>(BindingContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }

            context.BindingParameters.Add(this);
            return context.BuildInnerChannelListener<TChannel>();
        }

        public override bool CanBuildChannelListener<TChannel>(BindingContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }

            context.BindingParameters.Add(this);
            return context.CanBuildInnerChannelListener<TChannel>();
        }

        /// <summary>
        /// Returns the types object requested, if it exists, from the appropriate 
        /// layer of the channel stack.
        /// </summary>
        /// <typeparam name="T">the types object for which the method is querying</typeparam>
        /// <param name="context">the binding context for the current binding element</param>
        /// <returns>the typed object if present otherwise null</returns>
        public override T GetProperty<T>(BindingContext context)
        {
            if (typeof(T) == typeof(XmlDictionaryReaderQuotas))
            {
                return (T)(object)readerQuotas;
            }

            return base.GetProperty<T>(context);
        }

        #region IWsdlExportExtension Members

        void IWsdlExportExtension.ExportContract(WsdlExporter exporter, WsdlContractConversionContext context)
        {
        }

        void IWsdlExportExtension.ExportEndpoint(WsdlExporter exporter, WsdlEndpointConversionContext context)
        {
            // The MessageEncodingBindingElement is responsible for ensuring that the WSDL has the correct
            // SOAP version. We can delegate to the WCF implementation of TextMessageEncodingBindingElement for this.
            TextMessageEncodingBindingElement mebe = new TextMessageEncodingBindingElement {MessageVersion = messageVersion};
            ((IWsdlExportExtension)mebe).ExportEndpoint(exporter, context);
        }

        #endregion IWsdlExportExtension Members
    }
}
