﻿//-----------------------------------------------------------------------
// <copyright file="SimpleTextMessageEncoderFactory.cs" company="Eclectrics, LLC">
//     Copyright © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
//
// The intellectual property contained in this class belongs to Microsoft.
//  
// This document is free for all uses under the Creative Commons Attribution 
// 3.0 United Stated License.
//-----------------------------------------------------------------------

// This class uses code described here:
// http://msdn.microsoft.com/en-us/library/ms751486(VS.85).aspx

namespace SimpleTextMessageEncoderLibrary
{
    using System.ServiceModel.Channels;

    /// <summary>
    /// A MessageEncoderFactory derivative that knows how to create SimpleTextMessageEncoders.
    /// </summary>
    public class SimpleTextMessageEncoderFactory : MessageEncoderFactory
    {
        /// <summary>
        /// The MessageEncoder used created by the factory
        /// </summary>
        private readonly MessageEncoder encoder;

        /// <summary>
        /// The MessageVersion used to create the encoder
        /// </summary>
        private readonly MessageVersion version;

        /// <summary>
        /// The media type used to create the encoder
        /// </summary>
        private readonly string mediaType;

        /// <summary>
        /// The char set used to create the encoder
        /// </summary>
        private readonly string charSet;

        /// <summary>
        /// Initializes a new instance of the SimpleTextMessageEncoderFactory class.
        /// The factory knows how to create SimpleMessageEncoders that can read and 
        /// write to a stream for various types of message encoding
        /// </summary>
        /// <param name="mt">media type for encoder</param>
        /// <param name="cs">char set for encoder</param>
        /// <param name="ver">MessageVersion for the encoder</param>
        internal SimpleTextMessageEncoderFactory(string mt, string cs, MessageVersion ver)
        {
            this.mediaType = mt;
            this.charSet = cs;
            this.version = ver;
            this.encoder = new SimpleTextMessageEncoder(this);
        }

        /// <summary>
        /// Gets the MessageEncoder created by this factory.
        /// </summary>
        public override MessageEncoder Encoder 
        { 
            get { return this.encoder; } 
        }

        /// <summary>
        /// Gets the MessageVersion used to create the encoder.
        /// </summary>
        public override MessageVersion MessageVersion 
        { 
            get { return this.version; } 
        }

        /// <summary>
        /// Gets the media type used to create the encoder.
        /// </summary>
        public string MediaType 
        { 
            get { return this.mediaType; } 
        }

        /// <summary>
        /// Gets the char set used to create the encoder
        /// </summary>
        public string CharSet 
        { 
            get { return this.charSet; } 
        }
    }
}
