﻿//-----------------------------------------------------------------------
// <copyright file="MainForm.cs" company="Eclectrics, LLC">
//     Copyright © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
//
// Free for all uses under the Creative Commons Attribution 3.0 United 
// Stated License.
//-----------------------------------------------------------------------

namespace TestFromStateStation
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Diagnostics;
    using System.Windows.Forms;

    using Eclectrics.NationalWeatherServiceLibrary;

    /// <summary>
    /// The main application window.
    /// </summary>
    public partial class MainForm : Form
    {
        /// <summary>
        /// Notes that there is an outstanding request for a stations update.
        /// This happens when the user selects a state while the background
        /// worker is already busy getting stations for their previous selection.
        /// </summary>
        private bool pendingStationsUpdate;

        /// <summary>
        /// Used to signal when the stationsBackgroundWorker is busy, a little
        /// simpler than using BackgroundWorker.IsBusy.
        /// </summary>
        private bool stationsWorkerBusy;

        /// <summary>
        /// Notes that there is an outstanding request for current conditions.
        /// This happens when the user selects a station while the background
        /// worker is already busy getting the current conditions for their 
        /// previous selection.
        /// </summary>
        private bool pendingCurrentConditionsUpdate;

        /// <summary>
        /// Used to signal when the currentConditionsBackgroundWorker is busy,
        /// a little simpler than using BackgroundWorker.IsBusy.
        /// </summary>
        private bool currentConditionsWorkerBusy;

        /// <summary>
        /// Used to restore the current conditions for the selected station when
        /// the user selects Refresh from the Stations Cache drop down menu on
        /// the status bar.
        /// </summary>
        private string currentStationId = string.Empty;

        /// <summary>
        /// Initializes a new instance of the MainForm class.
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
            this.VerboseDebug = true;
        }

        /// <summary>
        /// Gets or sets a value indicating whether [verbose debug].
        /// </summary>
        /// <value>
        ///   <c>true</c> if [verbose debug]; otherwise, <c>false</c>.
        /// </value>
        public bool VerboseDebug { get; set; }

        /// <summary>
        /// Setup background workers, initialize UI.
        /// </summary>
        /// <param name="e">event args - not used</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            this.stationsBackgroundWorker.DoWork += StationsBackgroundWorkerDoWork;
            this.stationsBackgroundWorker.RunWorkerCompleted += this.StationsBackgroundWorkerRunWorkerCompleted;

            this.currentConditionsbackgroundWorker.DoWork += CurrentConditionsbackgroundWorkerDoWork;
            this.currentConditionsbackgroundWorker.RunWorkerCompleted += this.CurrentConditionsbackgroundWorkerRunWorkerCompleted;

            // Load the states combo box
            var states = ObservationStations.GetStates();
            foreach (var state in states)
            {
                this.statesComboBox.Items.Add(state);
            }

            this.UpdateCacheAvailable();
        }

        /// <summary>
        /// CC-Step 3: Startup method for the current conditions background worker.
        /// </summary>
        /// <param name="sender">sender - not used</param>
        /// <param name="e">event args - contains requested station id and holds 
        /// current conditions string result</param>
        private static void CurrentConditionsbackgroundWorkerDoWork(object sender, DoWorkEventArgs e)
        {
            e.Result = GetCurrentConditions(e.Argument as string);
        }

        /// <summary>
        /// CC-Step 4: Get the current weather conditions for an observation station.
        /// </summary>
        /// <param name="stationId">stationId where conditions are requested</param>
        /// <returns>an Xml string containing the current conditions or null 
        /// if an error occurred</returns>
        private static string GetCurrentConditions(string stationId)
        {
            var result = string.Empty;

            if (stationId != null)
            {
                string status;
                string doc;

                if (CurrentObservations.GetCurrentXmlObservationFromStationId(stationId, out doc, out status))
                {
                    result = doc;
                }
            }

            return result;
        }

        /// <summary>
        /// OS-Step 3: DoWork implementation for stations background worker.
        /// </summary>
        /// <param name="sender">sender - not used</param>
        /// <param name="e">event args - contains requested state and holds stations list result</param>
        private static void StationsBackgroundWorkerDoWork(object sender, DoWorkEventArgs e)
        {
            e.Result = UpdateStations(e.Argument as string);
        }

        /// <summary>
        /// OS-Step 4: Used from the DoWork method for _stationsBackgroundWorker - gets the list
        /// of observation stations for the requested state.
        /// </summary>
        /// <param name="stateAbbreviation">state for which to get observation stations</param>
        /// <returns>a list of stations</returns>
        private static IList<ObservationStation> UpdateStations(string stateAbbreviation)
        {
            IList<ObservationStation> stations;
            string status;

            ObservationStations.GetStations(stateAbbreviation, out stations, out status);
            return stations;
        }

        /// <summary>
        /// CC-Step 1: The user clicks on a station in the stations combo box - start the task
        /// of getting the current conditions.
        /// </summary>
        /// <param name="sender">sender - not used</param>
        /// <param name="e">event args - not used</param>
        private void StationsComboBoxSelectionChangeCommitted(object sender, EventArgs e)
        {
            var station = this.stationsComboBox.SelectedItem as ObservationStation;

            if (station != null)
            {
                this.StartWorkerGetCurrentConditions(station.StationId);
            }
        }

        /// <summary>
        /// CC-Step 2: Check for busy and if so, set pending, otherwise startup the
        /// currentConditionsBackgroundWorker.
        /// </summary>
        /// <param name="stationId">The station Id.</param>
        private void StartWorkerGetCurrentConditions(string stationId)
        {
            if (this.currentConditionsWorkerBusy)
            {
                this.pendingCurrentConditionsUpdate = true;
                return;
            }

            this.currentConditionsWorkerBusy = true;

            this.statusLabel.Text = Properties.Resources.WaitingForCurrentConditions;
            this.Cursor = Cursors.WaitCursor;

            this.currentConditionsbackgroundWorker.RunWorkerAsync(stationId);
        }

        /// <summary>
        /// CC-Step 5: When the current conditions background worker completes - update the current
        /// observations tree.
        /// </summary>
        /// <param name="sender">sender - not used</param>
        /// <param name="e">contains error info and current observations for the 
        /// requested station</param>
        private void CurrentConditionsbackgroundWorkerRunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(this, e.Error.Message);
                this.statusLabel.Text = e.Error.Message;
            }
            else if (e.Cancelled)
            {
                MessageBox.Show(this, Properties.Resources.GetCurrentConditionsCanceled);
            }
            else
            {
                this.currentObsTree.LoadXml(e.Result as string);
                this.statusLabel.Text = Properties.Resources.Ready;

                if (this.VerboseDebug)
                {
                    Debug.WriteLine("CurrentConditions background worker completed successfully.");
                }
            }

            this.Cursor = Cursors.Default;

            this.currentConditionsWorkerBusy = false;

            // In case the user selected another state while we were getting the stations
            // from the previous selection - fire off another refresh stations task.
            if (this.pendingCurrentConditionsUpdate && 
                this.stationsComboBox.SelectedItem != null && 
                this.stationsComboBox.SelectedItem is ObservationStation)
            {
                this.StartWorkerRefreshStations((this.stationsComboBox.SelectedItem as ObservationStation).StationId);

                if (this.VerboseDebug)
                {
                    var observationStation = this.statesComboBox.SelectedItem as ObservationStation;
                    if (observationStation != null)
                    {
                        Debug.WriteLine("Pending current conditions update started: " + observationStation.StationName);
                    }
                }
            }

            this.pendingStationsUpdate = false;
        }

        /// <summary>
        /// OS-Step 1: When the user clicks on a state in the states combo box, update
        /// the stations for that state.
        /// </summary>
        /// <param name="sender">sender - not used</param>
        /// <param name="e">event args - not used</param>
        private void StatesComboBoxSelectionChangeCommitted(object sender, EventArgs e)
        {
            this.currentStationId = string.Empty;

            if (this.statesComboBox.SelectedItem != null && this.statesComboBox.SelectedItem is StateInfo)
            {
                this.StartWorkerRefreshStations((this.statesComboBox.SelectedItem as StateInfo).Abbreviation);
            }
        }

        /// <summary>
        /// OS-Step 2: Start the task of updating the stations list for the specified state using
        /// a background worker.
        /// </summary>
        /// <param name="stateAbbreviation">The state abbreviation.</param>
        private void StartWorkerRefreshStations(string stateAbbreviation)
        {
            if (this.stationsWorkerBusy)
            {
                this.pendingStationsUpdate = true;
                return;
            }

            this.currentObsTree.Clear();
            this.stationsComboBox.SelectedItem = null;
            this.stationsComboBox.Items.Clear();

            this.stationsWorkerBusy = true;

            this.refreshButton.Enabled = false;

            this.statusLabel.Text = Properties.Resources.LoadingObservationStations;
            this.Cursor = Cursors.WaitCursor;

            this.stationsBackgroundWorker.RunWorkerAsync(stateAbbreviation);
        }

        /// <summary>
        /// OS-Step 5: When the stations background worker completes - update the stations combo box.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.ComponentModel.RunWorkerCompletedEventArgs"/> instance containing the event data.</param>
        private void StationsBackgroundWorkerRunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(this, e.Error.Message);
                statusLabel.Text = e.Error.Message;
            }
            else if (e.Cancelled)
            {
                MessageBox.Show(this, Properties.Resources.GetStationsCanceled);
            }
            else
            {
                this.UpdateStationsCombo(e.Result as IList<ObservationStation>);
                this.statusLabel.Text = Properties.Resources.Ready;

                if (this.VerboseDebug)
                {
                    Debug.WriteLine("StationsBackgroundWorker completed successfully.");
                }
            }

            this.Cursor = Cursors.Default;

            this.stationsWorkerBusy = false;

            // In case the user selected another state while we were getting the stations
            // from the previous selection - fire off another refresh stations task.
            if (this.pendingStationsUpdate && this.statesComboBox.SelectedItem != null && this.statesComboBox.SelectedItem is StateInfo)
            {
                this.StartWorkerRefreshStations((this.statesComboBox.SelectedItem as StateInfo).Abbreviation);

                if (this.VerboseDebug)
                {
                    var stateInfo = this.statesComboBox.SelectedItem as StateInfo;
                    if (stateInfo != null)
                    {
                        Debug.WriteLine("Pending observation stations update started: " + stateInfo.Name);
                    }
                }
            }

            this.UpdateCacheAvailable();
            this.refreshButton.Enabled = true;
            this.pendingStationsUpdate = false;
        }

        /// <summary>
        /// Update the stations combo box from the supplied list of stations.
        /// </summary>
        /// <param name="stations">list of stations</param>
        private void UpdateStationsCombo(IEnumerable<ObservationStation> stations)
        {
            int refreshIndex = -1;

            foreach (ObservationStation station in stations)
            {
                int index = this.stationsComboBox.Items.Add(station);

                // the refreshed list has the current station in it.
                if (!string.IsNullOrEmpty(this.currentStationId) && station.StationId == this.currentStationId)
                {
                    refreshIndex = index;
                }
            }

            // If this was a refresh - refresh the current observation for the selected station.
            if (refreshIndex >= 0)
            {
                this.stationsComboBox.SelectedIndex = refreshIndex;
                if (this.stationsComboBox.SelectedItem != null)
                {
                    var observationStation = this.stationsComboBox.SelectedItem as ObservationStation;
                    if (observationStation != null)
                    {
                        this.StartWorkerGetCurrentConditions(observationStation.StationId);
                    }
                }
            }

            this.currentStationId = string.Empty;
        }

        /// <summary>
        /// When the user selects the Refresh menu item from the status bar - flushes
        /// the observation stations cache and reloads the stations for the currently
        /// selected state.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void RefreshToolStripMenuItemClick(object sender, EventArgs e)
        {
            this.refreshButton.Enabled = false;

            // If there is a station selected, remember it so we can refresh its current
            // conditions when the stations refresh is complete.
            var observationStation = this.stationsComboBox.SelectedItem as ObservationStation;
            if (observationStation != null)
            {
                this.currentStationId = this.stationsComboBox.SelectedItem != null
                                            ? observationStation.StationId
                                            : string.Empty;
            }

            this.stationsComboBox.SelectedItem = null;
            this.stationsComboBox.Items.Clear();
            this.currentObsTree.Clear();

            string result;
            ObservationStations.FlushObservationFileCache(out result);

            this.UpdateCacheAvailable();

            if (this.statesComboBox.SelectedItem != null && this.statesComboBox.SelectedItem is StateInfo)
            {
                this.StartWorkerRefreshStations((this.statesComboBox.SelectedItem as StateInfo).Abbreviation);
            }
        }

        /// <summary>
        /// When the user selects the Clear menu item from the status bar - clears
        /// the observation stations cache and the UI.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ClearToolStripMenuItemClick(object sender, EventArgs e)
        {
            string result;
            ObservationStations.FlushObservationFileCache(out result);
            this.statesComboBox.SelectedItem = null;
            this.stationsComboBox.SelectedItem = null;
            this.stationsComboBox.Items.Clear();
            this.currentObsTree.Clear();

            this.UpdateCacheAvailable();
        }

        /// <summary>
        /// Update the image shown on the Stations Cache drop down menu on the status
        /// bar based on whether we have cached the stations list in isolated storage.
        /// </summary>
        private void UpdateCacheAvailable()
        {
            bool have = ObservationStations.HaveStations;
            this.stationsCacheDropDownButton.Image = have
                                                         ? Properties.Resources.StationsAvailable
                                                         : Properties.Resources.StationsEmpty;
            this.stationsCacheDropDownButton.ToolTipText = have
                                                                ? Properties.Resources.StationsListIsLocal
                                                                : Properties.Resources.StationsListNotDownloaded;
        }
    }
}
