﻿//-----------------------------------------------------------------------
// <copyright file="MainForm.cs" company="Eclectrics, LLC">
//     Copyright © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
//
// Free for all uses under the Creative Commons Attribution 3.0 United 
// Stated License.
//-----------------------------------------------------------------------

namespace TestFromZip
{
    using System;
    using System.ComponentModel;
    using System.Windows.Forms;

    using Eclectrics.NationalWeatherServiceLibrary;

    /// <summary>
    /// This is the application's main form.
    /// </summary>
    public partial class MainForm : Form
    {
        /// <summary>
        /// Defines the application states, used for updating the UI
        /// </summary>
        private enum UIState { EmptyZip, PartialZip, InvalidZip, WaitingForCurrentConditions, GotCurrentConditions };

        /// <summary>
        /// Our local indicator of activity for the background worker thread
        /// </summary>
        private bool workerBusy;

        /// <summary>
        /// Used to note changes to zip code text box while backgroundWorker is busy
        /// </summary>
        private bool havePendingZipUpdate;

        /// <summary>
        /// Most recent pending zip value.
        /// </summary>
        private string pendingZip = string.Empty;

        /// <summary>
        /// Initialize an instance of the MainForm class.
        /// </summary>
        public MainForm()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Setup the background worker and Initialize the UI.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            zipTextBox.ZipChanged += this.ZipTextBoxZipChanged;

            backgroundWorker.DoWork += this.BackgroundWorkerDoWork;
            backgroundWorker.RunWorkerCompleted += this.BackgroundWorkerRunWorkerCompleted;

            statusLabel.Text = Properties.Resources.ENTER_ZIP;
        }

        /// <summary>
        /// When the zip code is complete, get the current conditions. Maps ZipState
        /// to UIState.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="TestFromZip.ZipChangedEventArgs"/> instance containing the event data.</param>
        private void ZipTextBoxZipChanged(object sender, ZipChangedEventArgs e)
        {
            if (e.State == ZipState.Complete)
            {
                UpdateUI(UIState.WaitingForCurrentConditions, null);
                if (workerBusy)
                {
                    havePendingZipUpdate = true;
                    pendingZip = e.ZipCode;
                }
                else
                {
                    StartWorker(e.ZipCode);
                }
            }
            else if (e.State == ZipState.Empty)
            {
                UpdateUI(UIState.EmptyZip, null);
            }
            else if (e.State == ZipState.Partial)
            {
                UpdateUI(UIState.PartialZip, null);
            }
            else if (e.State == ZipState.Invalid)
            {
                UpdateUI(UIState.InvalidZip, null);
            }
        }

        /// <summary>
        /// Request current weather conditions by handing the task off to the
        /// background worker.
        /// </summary>
        /// <param name="zip">The requested zip code.</param>
        private void StartWorker(string zip)
        {
            if (!this.workerBusy)
            {
                this.workerBusy = true;
                this.havePendingZipUpdate = false;

                UpdateUI(UIState.WaitingForCurrentConditions, null);

                this.backgroundWorker.RunWorkerAsync(zip);
            }
        }

        /// <summary>
        /// Gets the current weather conditions for the supplied zip code. This method
        /// is not called on the UI thread - do not modify control from here.
        /// </summary>
        /// <param name="sender">sender - not used</param>
        /// <param name="e">contains the zip code in the Argument member</param>
        private void BackgroundWorkerDoWork(object sender, DoWorkEventArgs e)
        {
            e.Result = GetCurrentConditions(e.Argument as string);
        }

        /// <summary>
        /// Updates UI to reflect results from background worker. Called upon 
        /// completion of background worker task. This method is called on the
        /// UI thread so we can modify controls here.
        /// </summary>
        /// <param name="sender">sender - not used</param>
        /// <param name="e">contains a GetCurrentConditionsResult in the Results member</param>
        private void BackgroundWorkerRunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            var result = e.Result as GetCurrentConditionsResult;
            if (result != null && result.Success)
            {
                this.xmlTree.LoadXml(result.CurrentConditions);
                this.distanceLabel.Text = Properties.Resources.DistanceToStation + result.DistanceToStation.ToString("F2") + Properties.Resources.KilometersAbbreviation;
                this.stationNameLabel.Text = Properties.Resources.Station + result.StationName;
                UpdateUI(UIState.GotCurrentConditions, null);
            }
            else
            {
                if (result != null)
                {
                    this.UpdateUI(UIState.InvalidZip, result.Status);
                }
            }

            this.workerBusy = false;

            if (this.havePendingZipUpdate)
            {
                StartWorker(this.pendingZip);
            }
        }

        /// <summary>
        /// Update the UI state
        /// </summary>
        /// <param name="state">new UI state</param>
        /// <param name="status">additional status text for status bar</param>
        private void UpdateUI(UIState state, string status)
        {
            switch (state)
            {
                case UIState.EmptyZip:
                    statusLabel.Text = (string.IsNullOrEmpty(status) ? Properties.Resources.ENTER_ZIP : status);
                    progressBar.Enabled = progressBar.Visible = false;
                    break;
                case UIState.PartialZip:
                    statusLabel.Text = Properties.Resources.ENTER_ZIP;
                    stationNameLabel.Text = string.Empty;
                    distanceLabel.Text = string.Empty;
                    xmlTree.Clear();
                    break;
                case UIState.InvalidZip:
                    statusLabel.Text = Properties.Resources.INVALID_ZIP;
                    progressBar.Enabled = progressBar.Visible = false;
                    stationNameLabel.Text = string.Empty;
                    distanceLabel.Text = string.Empty;
                    xmlTree.Clear();
                    break;
                case UIState.WaitingForCurrentConditions:
                    statusLabel.Text = Properties.Resources.WAITING_FOR_CURCON;
                    progressBar.Enabled = progressBar.Visible = true;
                    break;
                case UIState.GotCurrentConditions:
                    statusLabel.Text = Properties.Resources.READY;
                    progressBar.Enabled = progressBar.Visible = false;
                    break;
            }
        }

        /// <summary>
        /// The DoWork method for the background worker.
        /// </summary>
        /// <param name="zipCode">Zip Code for requested location</param>
        /// <returns>detailed description of the result</returns>
        private GetCurrentConditionsResult GetCurrentConditions(string zipCode)
        {
            string currentObsDoc;
            string stationName;
            string status;
            double distance;

            bool success = CurrentObservations.GetCurrentXmlObservationFromZip(zipCode, out currentObsDoc, out stationName, out distance, out status);

            return new GetCurrentConditionsResult(success, currentObsDoc, stationName, distance, status);
        }
    }
}
