﻿//-----------------------------------------------------------------------
// <copyright file="ZipCodeTextBox.cs" company="Eclectrics, LLC">
//     Copyright © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
//
// Free for all uses under the Creative Commons Attribution 3.0 United 
// Stated License.
//-----------------------------------------------------------------------

namespace TestFromZip
{
    using System;
    using System.Linq;
    using System.Media;
    using System.Windows.Forms;

    /// <summary>
    /// A TextBox derivative that understands zip codes.
    /// </summary>
    internal class ZipCodeTextBox : TextBox
    {
        /// <summary>
        /// Length of standard zip code.
        /// </summary>
        private const int StdZipLength = 5;

        /// <summary>
        /// Describes the signature for the handler of the event fired
        /// when a complete zip code has been entered.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="TestFromZip.ZipChangedEventArgs"/> instance containing the event data.</param>
        public delegate void ZipChangedEventHandler(object sender, ZipChangedEventArgs e);

        /// <summary>
        /// Occurs when [zip changed].
        /// </summary>
        public event ZipChangedEventHandler ZipChanged;

        /// <summary>
        /// Gets the current status of the text in the zipTextBox
        /// </summary>
        public ZipState State
        {
            get
            {
                // Test for no zip text.
                if (this.Text.Length == 0)
                {
                    return ZipState.Empty;
                }

                // Test for incomplete text - too few characters.
                if (this.Text.Length < StdZipLength)
                {
                    if (this.ZipIsAllDigits)
                    {
                        return ZipState.Partial;
                    }

                    return ZipState.Invalid;
                }

                // Test for correct number of characters.
                if (this.Text.Length == StdZipLength)
                {
                    return this.ZipIsAllDigits ? ZipState.Complete : ZipState.Invalid;
                }

                // Otherwise it's an error.
                return ZipState.Invalid;
            }
        }

        /// <summary>
        /// Gets a value indicating whether all characters in the zipTextBox are digits.
        /// </summary>
        private bool ZipIsAllDigits
        {
            get
            {
                return this.Text.All(char.IsDigit);
            }
        }

        /// <summary>
        /// Occurs when the text changes.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> that contains the event data.</param>
        protected override void OnTextChanged(EventArgs e)
        {
            base.OnTextChanged(e);

            if (this.ZipChanged != null)
            {
                this.ZipChanged(this, new ZipChangedEventArgs(this.State, this.Text));
            }
        }

        /// <summary>
        /// Check to see if we have reached the limit for character count, and 
        /// if the key pressed is a digit. If we can accept more characters and 
        /// the key was a digit, allow the event, otherwise eat the key press.
        /// </summary>
        /// <param name="e">Event args - e.KeyChar used</param>
        protected override void OnKeyPress(KeyPressEventArgs e)
        {
            base.OnKeyPress(e);

            // If we are waiting on more characters, evaluate the key press
            if (this.Text.Length < StdZipLength)
            {
                if (char.IsDigit(e.KeyChar) || char.IsControl(e.KeyChar))
                {
                }
                else
                {
                    OnError(ref e);
                }
            }        
            else if (char.IsControl(e.KeyChar))
            {
                // If we have enough characters, allow editing.
            }      
            else
            {
                // But don't allow entering more characters
                OnError(ref e);
            }
        }

        /// <summary>
        /// Performs error actions when key press was invalid.
        /// </summary>
        /// <param name="e">The <see cref="System.Windows.Forms.KeyPressEventArgs"/> instance containing the event data.</param>
        private static void OnError(ref KeyPressEventArgs e)
        {
            SystemSounds.Exclamation.Play();
            e.Handled = true;
        }
    }
}
