﻿//-----------------------------------------------------------------------------
// <copyright file="XmlTreeView.cs" company="Eclectrics, LLC">
//     © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
// Free for all uses under the Creative Commons Attribution 3.0 United States License
// http://creativecommons.org/licenses/by/3.0/us/
//-----------------------------------------------------------------------------

namespace Eclectrics.XmlToolsLibrary
{
    using System;
    using System.Text;
    using System.Windows.Forms;
    using System.Xml;

    /// <summary>
    /// A simple control to present Xml strings in a readable tree view. Hides
    /// an underlying TreeView, only exposing methods that support the illusion
    /// we have a fully-custom XmlTree control.
    /// </summary>
    public partial class XmlTreeView : UserControl
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="XmlTreeView"/> class.
        /// </summary>
        public XmlTreeView()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Load the tree from an xml string. The tree is cleared before nodes are added.
        /// </summary>
        /// <param name="xmlString">Xml string that will be loaded into the tree</param>
        public void LoadXml(string xmlString)
        {
            // This is important - it stops the control from painting while we are adding
            // nodes - but be sure to call EndUpdate() or your toast.
            this.treeViewBase.BeginUpdate();

            try
            {
                var doc = new XmlDocument();
                doc.LoadXml(xmlString);

                this.treeViewBase.Nodes.Clear();

                if (doc.DocumentElement != null)
                {
                    var rootNode = new TreeNode();
                    var details = new XmlTreeNodeDetails();
                    rootNode.Tag = details;

                    var rootText = new StringBuilder();
                    rootText.Append(doc.DocumentElement.Name);
                    details.ElementTag = doc.DocumentElement.Name;
                    this.AppendAttributes(doc.DocumentElement, ref rootText, ref details);
                    UpdateNodeWidth(details);

                    rootNode.Text = rootText.ToString();

                    this.treeViewBase.Nodes.Add(rootNode);
                    this.AddNode(doc.DocumentElement, ref rootNode);

                    this.treeViewBase.ExpandAll();
                    rootNode.EnsureVisible();
                }
            }
            catch (XmlException xmlEx)
            {
                MessageBox.Show(xmlEx.Message);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                this.treeViewBase.EndUpdate();
                this.treeViewBase.Refresh();
            }
        }

        /// <summary>
        /// Clear the entire tree.
        /// </summary>
        public void Clear()
        {
            if (this.treeViewBase != null)
            {
                this.treeViewBase.SuspendLayout();
                this.treeViewBase.Nodes.Clear();
                this.treeViewBase.ResumeLayout();
            }
        }

        /// <summary>
        /// Updates the width of the node.
        /// </summary>
        /// <param name="details">The details.</param>
        private static void UpdateNodeWidth(XmlTreeNodeDetails details)
        {
            details.WidthIsDirty = true;
        }

        /// <summary>
        /// Recurse through nodes, adding children to the tree.
        /// </summary>
        /// <param name="parentXmlNode">The parent XML node.</param>
        /// <param name="parentTreeNode">The parent tree node.</param>
        private void AddNode(XmlNode parentXmlNode, ref TreeNode parentTreeNode)
        {
            if (parentXmlNode.HasChildNodes)
            {
                var xmlNodeList = parentXmlNode.ChildNodes;
                foreach (XmlNode childXmlNode in xmlNodeList)
                {
                    if (childXmlNode.NodeType == XmlNodeType.Text)
                    {
                        if (parentTreeNode.Tag is XmlTreeNodeDetails)
                        {
                            // This is where we add the element content.
                            var details = parentTreeNode.Tag as XmlTreeNodeDetails;

                            // Add a space before the content - it looks better.
                            string content = " " + childXmlNode.InnerText;
                            parentTreeNode.Text += content;
                            details.Content = content;
                            UpdateNodeWidth(details);
                        }
                    }
                    else
                    {
                        var childTreeNode = new TreeNode();
                        var details = new XmlTreeNodeDetails();
                        childTreeNode.Tag = details;

                        // This is where we add the tag.
                        var tag = new StringBuilder();
                        tag.Append(childXmlNode.Name + ":");
                        details.ElementTag = tag.ToString();
                        UpdateNodeWidth(details);

                        this.AppendAttributes(childXmlNode, ref tag, ref details);
                        childTreeNode.Text = tag.ToString();

                        parentTreeNode.Nodes.Add(childTreeNode);

                        this.AddNode(childXmlNode, ref childTreeNode);
                    }
                }
            }
        }

        /// <summary>
        /// Appends any attributes of the supplied XmlNode to the supplied StringBuilder instance.
        /// </summary>
        /// <param name="xmlNode">node whose attributes will be added to the supplied string</param>
        /// <param name="treeNodeText">text to which attributes will be appended</param>
        /// <param name="details">The attribute details.</param>
        private void AppendAttributes(XmlNode xmlNode, ref StringBuilder treeNodeText, ref XmlTreeNodeDetails details)
        {
            if (xmlNode == null)
            {
                throw new ArgumentNullException("xmlNode");
            }

            if (treeNodeText == null)
            {
                throw new ArgumentNullException("treeNodeText");
            }

            if (details == null)
            {
                throw new ArgumentNullException("details");
            }

            if (xmlNode.Attributes == null)
            {
                return;
            }

            // This is where we implement the visual style for attributes.
            foreach (XmlAttribute attribute in xmlNode.Attributes)
            {
                var newAttribute = new StringBuilder();

                // Add a space before the attribute - it looks better.
                newAttribute.Append(" [" + attribute.Name + " = " + attribute.InnerText + "]");
                details.Attributes.Add(newAttribute.ToString());
                treeNodeText.Append(newAttribute);
                UpdateNodeWidth(details);
            }
        }
    }
}