﻿//-----------------------------------------------------------------------------
// <copyright file="XmlTreeViewBase.cs" company="Eclectrics, LLC">
//     © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
// Free for all uses under the Creative Commons Attribution 3.0 United States License
// http://creativecommons.org/licenses/by/3.0/us/
//-----------------------------------------------------------------------------

namespace Eclectrics.XmlToolsLibrary
{
    using System.Drawing;
    using System.Drawing.Text;
    using System.Linq;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;

    /// <summary>
    /// Derived from TreeView so we can do custom drawing - does a respectable
    /// job of flicker-free drawing.  Only knows how to draw in our one-and-only
    /// custom style.
    /// </summary>
    internal class XmlTreeViewBase : TreeView
    {
        #region Fields

        /// <summary>
        /// Attribute text color when node is not selected
        /// </summary>
        private readonly Color attributeColor = Color.Red;

        /// <summary>
        /// attribute text color when node is selected.
        /// </summary>
        private readonly Color highlightAttributeColor = Color.LightSalmon;

        /// <summary>
        /// content text color when node is not selected.
        /// </summary>
        private readonly Color contentColor = Color.Green;

        /// <summary>
        /// content text color when node is selected.
        /// </summary>
        private readonly Color highlightContentColor = Color.LawnGreen;

        /// <summary>
        /// tag text color when node is not selected.
        /// </summary>
        private readonly Color tagColor = Color.Blue;

        /// <summary>
        /// tag text color when node is selected.
        /// </summary>
        private readonly Color highlightTagColor = Color.Cyan;

        /// <summary>
        /// Focused node background when the tree view doesn't have focus.
        /// </summary>
        private readonly Color noFocusHighlightColor = SystemColors.InactiveCaption;

        /// <summary>
        /// The padding around text.
        /// </summary>
        private readonly Padding nodeTextPad = new Padding(2);

        /// <summary>
        /// The brush used when the control does not have focus.
        /// </summary>
        private SolidBrush noFocusBkBrush;

        /// <summary>
        /// The background brush. We use this a lot so create it once...
        /// </summary>
        private SolidBrush backBrush;

        /// <summary>
        /// The node that just received a mouse down event.
        /// </summary>
        private TreeNode mouseDownNode;

        /// <summary>
        /// The format used to paint text.
        /// </summary>
        private StringFormat format;

        #endregion Fields

        /// <summary>
        /// Raises the <see cref="M:System.Windows.Forms.Control.CreateControl"/> method.
        /// </summary>
        protected override void OnCreateControl()
        {
            base.OnCreateControl();

            this.format = StringFormat.GenericTypographic;
            this.format.FormatFlags |= StringFormatFlags.MeasureTrailingSpaces;

            // Don't need this when doing UserPaint, uncomment to try DrawModes
            // this.DrawMode = TreeViewDrawMode.OwnerDrawAll;

            // Need to set these three style bits to get flicker-free drawing, 
            // comment-out to try DrawModes.
            this.SetStyle(ControlStyles.UserPaint, true);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer, true);

            this.backBrush = new SolidBrush(this.BackColor);
            this.noFocusBkBrush = new SolidBrush(this.noFocusHighlightColor);
        }

        /// <summary>
        /// Releases the unmanaged resources used by the <see cref="T:System.Windows.Forms.TreeView"/> and optionally releases the managed resources.
        /// </summary>
        /// <param name="disposing">true to release both managed and unmanaged resources; false to release only unmanaged resources.</param>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (this.backBrush != null)
                {
                    this.backBrush.Dispose();
                    this.backBrush = null;
                }

                if (this.noFocusBkBrush != null)
                {
                    this.noFocusBkBrush.Dispose();
                    this.noFocusBkBrush = null;
                }
            }

            base.Dispose(disposing);
        }

        // Don't need this when doing UserPaint, uncomment to improve the redraw
        // flicker when using one of the TreeViewDrawModes.
        // protected override void WndProc(ref Message m)
        // {
        //    // It helps a bit with flicker to ignore erase background.
        //    const int WM_ERASEBKGND = 0x0014;
        //    if (m.Msg == WM_ERASEBKGND)
        //        return;
        //    base.WndProc(ref m);
        // }

        /// <summary>
        /// Override OnKeyUp to provide copy to clipboard support.
        /// </summary>
        /// <param name="e">The KeyEventArgs describing the event.</param>
        protected override void OnKeyUp(KeyEventArgs e)
        {
            base.OnKeyUp(e);

            if (e.Control && ((e.KeyCode & Keys.C) != 0) && this.SelectedNode != null)
            {
                Clipboard.SetText(this.SelectedNode.Text);
            }
        }

        /// <summary>
        /// Set _mouseDownNode if the user has clicked on a node.
        /// </summary>
        /// <param name="e">The MouseEventArgs describing the event.</param>
        protected override void OnMouseDown(MouseEventArgs e)
        {
            base.OnMouseDown(e);

            var hit = this.HitTest(e.Location);
            this.mouseDownNode = hit.Node;
        }

        /// <summary>
        /// Clear mouseDownNode.
        /// </summary>
        /// <param name="e">The MouseEventArgs describing the event.</param>
        protected override void OnMouseUp(MouseEventArgs e)
        {
            base.OnMouseUp(e);

            this.mouseDownNode = null;
        }

        /// <summary>
        /// All painting gets done here.
        /// </summary>
        /// <param name="e">The PaintEventArgs describing the event.</param>
        protected override void OnPaint(PaintEventArgs e)
        {
            // If we have nodes, then we have to draw them - and that
            // means everything about the node.
            if (this.Nodes.Count > 0)
            {
                // If we have nodes then there has to be a Nodes[0]. Use it to get the
                // height of one TreeNode - in a TreeView, all nodes are the same height
                // (unless you are using PInvoke to call TVM_SETITEM and set TV_ITEM 
                // iIntegral member for a specific node - which we are not doing!).
                int nodeHeight = this.Nodes[0].Bounds.Height;

                int curY = 0;

                // Walk down the client area, stepping by the height of one row, looking 
                // for nodes.
                while (curY <= this.ClientRectangle.Height)
                {
                    // This probably isn't the most efficient way to do this, but
                    // at least we aren't writing our own code to walk the tree...
                    TreeNode node = this.GetNodeAt(0, curY);

                    // If this rect does contain a node, draw the node.
                    if (node != null)
                    {
                        var args = new DrawTreeNodeEventArgs(
                            e.Graphics,
                            node,
                            e.ClipRectangle,
                            this.GetNodeStates(node)); // manufacture the proper TreeNodeStates

                        this.DrawCustomNode(ref args);
                    }
                    else
                    {
                        // If this rect doesn't contain a node, we need to fill it with the 
                        // background color.
                        e.Graphics.FillRectangle(
                            this.backBrush,
                            0,
                            curY,
                            this.ClientRectangle.Width,
                            nodeHeight);
                    }

                    curY += nodeHeight;
                }
            }
            else
            {
                // If there are no nodes in the tree, we're going to have to paint the 
                // entire background.
                e.Graphics.FillRectangle(this.backBrush, this.ClientRectangle);
            }

            // Don't need - in this case we can guarantee no one is listening...
            // base.OnPaint(e);
        }

        /// <summary>
        /// Paint the background - not useful in this implementation.
        /// </summary>
        /// <param name="pevent">The PaintEventArgs describing the event.</param>
        protected override void OnPaintBackground(PaintEventArgs pevent)
        {
            // We can do a better job of predicting where nodes will paint the background
            // and what area is left, so just disable background painting.  See OnPaint().
            // base.OnPaintBackground(pevent);
        }

        /// <summary>
        /// This is only called if we are implementing OwnerDrawText or OwnerDrawAll.
        /// It that's the case, then it's up to us to do the drawing.
        /// </summary>
        /// <param name="e">The DrawTreeNodeEventArgs describing the event.</param>
        protected override void OnDrawNode(DrawTreeNodeEventArgs e)
        {
            if (e.Node.Tag is XmlTreeNodeDetails)
            {
                this.DrawCustomNode(ref e);
            }
            else
            {
                e.DrawDefault = true;
            }

            base.OnDrawNode(e);
        }

        /// <summary>
        /// Draw the Xml element with our own style.
        /// </summary>
        /// <param name="e">The DrawTreeNodeEventArgs describing the event.</param>
        private void DrawCustomNode(ref DrawTreeNodeEventArgs e)
        {
            var details = e.Node.Tag as XmlTreeNodeDetails;
            if (details == null)
            {
                e.DrawDefault = true;
            }
            else
            {
                // Needed so MeasureText and DrawText position strings next to each
                // other correctly.
                e.Graphics.TextRenderingHint = TextRenderingHint.ClearTypeGridFit;

                // No sense drawing if there is nowhere to draw.
                if (e.Bounds.Width == 0)
                {
                    return;
                }

                RectangleF availableRect = e.Node.Bounds;

                // First time a node is selected, measure the width of the text so we
                // can highlight the background correctly when selected.
                if (details.WidthIsDirty)
                {
                    details.Width = (int)this.MeasureCustomNode(e.Graphics, details) + this.nodeTextPad.Horizontal;
                }

                // This is how the standard TreeView works - it paints highlighted
                // background on "Focused" not on Selected.
                if ((e.State & TreeNodeStates.Focused) != 0)
                {
                    // Paint the background for a focused node.

                    // Fill the background to the left of the node.
                    if (availableRect.Left - this.nodeTextPad.Left > 0)
                    {
                        e.Graphics.FillRectangle(
                            this.backBrush,
                            0,
                            availableRect.Top,
                            availableRect.Left - this.nodeTextPad.Left,
                            availableRect.Height);
                    }

                    // Highlight the node text by drawing the background in the system highlight color.
                    // Use the standard color for the highlight, but a more muted color if the tree
                    // does not have input focus.
                    e.Graphics.FillRectangle(
                        (this.Focused ? SystemBrushes.Highlight : this.noFocusBkBrush),
                        availableRect.Left - this.nodeTextPad.Left,
                        availableRect.Top,
                        details.Width + this.nodeTextPad.Horizontal,
                        availableRect.Height);

                    // Fill any remaining background to the right of the text.
                    if (availableRect.Left + details.Width + this.nodeTextPad.Right < this.ClientRectangle.Right)
                    {
                        e.Graphics.FillRectangle(
                            this.backBrush,
                            availableRect.Left + details.Width,
                            availableRect.Y,
                            this.ClientRectangle.Right - availableRect.Left,
                            availableRect.Height);
                    }
                }
                else
                {
                    // If the node is not focused, fill the background for the entire node using
                    // the entire ClientRectangle.Width.
                    e.Graphics.FillRectangle(
                        this.backBrush,
                        0,
                        e.Node.Bounds.Y,
                        this.ClientRectangle.Width,
                        e.Node.Bounds.Height);
                }

                // Draw the node expander/collapser symbols.  Only draw if this node has children.
                if (e.Node.GetNodeCount(false) > 0)
                {
                    string text = e.Node.IsExpanded ? "-" : "+";

                    using (var b = new SolidBrush(SystemColors.ControlDarkDark))
                    {
                        // Since e.Node.Bounds.X defines the left edge of the node text, which
                        // is indented by one TreeView.Indent, it is easy to position the +/- 
                        // as e.Node.Bounds.X - this.Indent.  This matches the location the
                        // underlying tree view is expecting so there is no need to override
                        // HitTest.
                        e.Graphics.DrawString(text, this.Font, b, e.Node.Bounds.X - this.Indent, e.Node.Bounds.Y);
                    }
                }

                // Draw the tag - always first on a line.
                if (!string.IsNullOrEmpty(details.ElementTag))
                {
                    this.DrawElement(
                        e.Graphics,
                        details.ElementTag,
                        ref availableRect,
                        ((e.State & TreeNodeStates.Focused) != 0 && this.Focused ? this.highlightTagColor : this.tagColor));
                }

                // Next come the attributes.
                foreach (string attr in details.Attributes)
                {
                    this.DrawElement(
                        e.Graphics,
                        attr,
                        ref availableRect,
                        ((e.State & TreeNodeStates.Focused) != 0 && this.Focused ? this.highlightAttributeColor : this.attributeColor));
                }

                // And finally the content.
                if (!string.IsNullOrEmpty(details.Content))
                {
                    this.DrawElement(
                        e.Graphics,
                        details.Content,
                        ref availableRect,
                        (e.State & TreeNodeStates.Focused) != 0 && this.Focused ? this.highlightContentColor : this.contentColor);
                }
            }
        }

        /// <summary>
        /// Draw a single stylized text span.
        /// </summary>
        /// <param name="g">The graphics context.</param>
        /// <param name="element">The element to draw.</param>
        /// <param name="availableRect">The space available.</param>
        /// <param name="foreColor">The foreground color.</param>
        private void DrawElement(Graphics g, string element, ref RectangleF availableRect, Color foreColor)
        {
            var requiredSize = g.MeasureString(element, this.Font, int.MaxValue, this.format);

            using (var b = new SolidBrush(foreColor))
            {
                g.DrawString(element, this.Font, b, availableRect.X, availableRect.Y, this.format);
            }

            // Reduce the available space by the amount we just used.
            availableRect.X += requiredSize.Width;
            availableRect.Width -= requiredSize.Width;
        }

        /// <summary>
        /// Compute the text width so we can draw the focused background correctly.
        /// </summary>
        /// <param name="g">The graphics context</param>
        /// <param name="details">The item to measure.</param>
        /// <returns>The length required.</returns>
        private float MeasureCustomNode(Graphics g, XmlTreeNodeDetails details)
        {
            float width = 0.0f;

            // The tag - always first on a line.
            if (!string.IsNullOrEmpty(details.ElementTag))
            {
                width += this.MeasureElement(g, details.ElementTag);
            }

            // Next come the attributes.
            width += details.Attributes.Sum(attr => this.MeasureElement(g, attr));

            // And finally the content.
            if (!string.IsNullOrEmpty(details.Content))
            {
                width += this.MeasureElement(g, details.Content);
            }

            details.WidthIsDirty = false;

            return width;
        }

        /// <summary>
        /// Measures the element.
        /// </summary>
        /// <param name="g">The graphics context.</param>
        /// <param name="element">The element to measure.</param>
        /// <returns>The required length.</returns>
        private float MeasureElement(Graphics g, string element)
        {
            return g.MeasureString(element, this.Font, int.MaxValue, this.format).Width;
        }

        /// <summary>
        /// Use PInvoke to access the underlying TreeViewCtrl to get the state of the 
        /// requested node. We need this for drawing the focused state.  All we need 
        /// in this implementation is focused and selected - so that's all we're producing.
        /// </summary>
        /// <param name="node">The tree node of interest.</param>
        /// <returns>The state of the requested tree node.</returns>
        private TreeNodeStates GetNodeStates(TreeNode node)
        {
            TreeNodeStates states = 0;

            // If mouse button is not down, then normal rules apply - if a node is
            // selected, then it also has focus. We set _mouseDownNode in our
            // OnMouseDown and clear it in our OnMouseUp overrides.
            if (this.mouseDownNode == null)
            {
                if (node.IsSelected)
                {
                    states = TreeNodeStates.Selected | TreeNodeStates.Focused;
                }
            }
            else if (node != this.mouseDownNode)
            {
                // But if a mouse button has been clicked on a node, the we need to check
                // to see if that node is currently drop-highlighted.

                // If this is the currently selected node, but a mouse button has been
                // clicked-and-held on another node and that node is currently showing 
                // the drop-highlight, then we don't show selection highlighting for this 
                // node.
                if (!this.IsDropHighlighted(this.mouseDownNode) && node.IsSelected)
                {
                    states = TreeNodeStates.Selected | TreeNodeStates.Focused;
                }
            }
            else if (node == this.mouseDownNode)
            {
                // Otherwise if this is the node that has been clicked-and-held and if
                // this node is currently showing drop-highlighting, then show it. We 
                // still need to check to see if drop-highlighting is being shown for 
                // this node because is the user clicked-and-held on this node, but
                // then started to move the mouse, drop-highlighting is removed by
                // the underlying Win32 tree control.
                if (this.IsDropHighlighted(node))
                {
                    states = TreeNodeStates.Focused;
                }
            }

            return states;
        }

        /// <summary>
        /// Drop-highlighting is a behavior from the underlying Win32 tree control.  For our
        /// purposes, drop-highlighting happens when the user click-and-holds on a tree 
        /// node.  If the mouse moves (more than a few pixels) while a node is drop-highlighted, 
        /// drop-highlighting is removed. This method reaches into the underlying tree control
        /// to determine if the supplied node is currently drop-highlighted.
        /// </summary>
        /// <param name="node">The node of interest.</param>
        /// <returns>True if highlighted, otherwise false.</returns>
        private bool IsDropHighlighted(TreeNode node)
        {
            const int MASK = NativeMethods.TVIF_STATE | NativeMethods.TVIF_HANDLE;

            // TV_ITEM stateMask doesn't seem to matter when reading state - we get
            // all the state bits anyway (Windows7).
            const int STATEMASK = NativeMethods.TVIS_DROPHILITED;

            NativeMethods.TV_ITEM lParam = new NativeMethods.TV_ITEM { hItem = node.Handle, mask = MASK, stateMask = STATEMASK };

            UnsafeNativeMethods.SendMessage(
                new HandleRef(this, this.Handle),
                NativeMethods.TVM_GETITEMW,
                0,
                ref lParam);

            return (lParam.state & NativeMethods.TVIS_DROPHILITED) != 0;
        }
    }
}

